
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

// Our own library
using XNACS1Lib;

#endregion


// 
//
// ALL Media files must be PART of the solution (to be deployed over to the XBOX)!!


namespace BlockBreaker_Program
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class BlockBreaker : XNACS1Base
    {
        public const float BLOCK_HEIGHT = 1.0f;
        public const float BLOCK_WIDTH = BLOCK_HEIGHT * 2.0f;
        private const float WORLD_MIN_X = 0.0f;
        private const float WORLD_MIN_Y = 0.0f;
        public  const float WORLD_WIDTH = BLOCK_WIDTH * NUM_BLOCK_COL;
        public const float WORLD_HEIGHT = WORLD_WIDTH * 1080.0f / 1920.0f;

        public const float LOWEST_BLOCK_Y = WORLD_HEIGHT * 0.45f;
        public const float HIGHEST_BLOCK_Y = WORLD_HEIGHT * 0.9f;
        public const int NUM_BLOCK_ROWS = (int) (HIGHEST_BLOCK_Y - LOWEST_BLOCK_Y);
        public const int NUM_BLOCK_COL = 15;

        public const float UNIT_SPEED = WORLD_WIDTH / 120.0f;

        private Paddle m_Paddle;
        private BallSet m_Balls;
        private BlockSet m_Blocks;

        private bool m_Paused;
        private bool m_Won;

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void InitializeWorld()
        {   
            World.SetWorldCoordinate(new Vector2(WORLD_MIN_X, WORLD_MIN_Y), WORLD_WIDTH);

            // initialize graphics
            World.SetBackgroundTexture(@"bgTexture");
                       
            // game state
            m_Paddle = new Paddle();
            m_Balls = new BallSet();
            m_Blocks = new BlockSet();

            m_Paused = false;
            m_Won = false;
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void UpdateWorld()
        {
            if (XNACS1Base.GamePad.ButtonBClicked())
            {
                if (m_Won)
                {
                    m_Balls.ClearBalls();
                }
                m_Won = false;
                m_Blocks.InitializeBlocks();
                World.SetBackgroundTexture(@"bgTexture");
            }

            if (m_Won)
                return;

            if (m_Blocks.HasWon())
            {
                World.SetBackgroundTexture("WinImage");
                PlayACue("Victory");
                m_Won = true;
                return;
            }
            if (XNACS1Base.GamePad.ButtonAClicked())
                m_Balls.AddNewBall(m_Paddle.Position);

            // Allows the default game to exit on Xbox 360 and Windows
            if (XNACS1Base.GamePad.ButtonBackClicked())
                this.Exit();

            Vector2 delta = XNACS1Base.GamePad.ThumbSticks.Right;
            m_Paddle.MovePaddle(delta.X);

            if (XNACS1Base.GamePad.ButtonStartClicked())
                m_Paused = !m_Paused;

            if (m_Paused)
                return;

            m_Balls.Update(m_Paddle, m_Blocks);         
        }

    }
}
