using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

// Our own library
using XNACS1Lib;

namespace BlockBreaker_Program
{
    class Block
    {
        protected XNACS1Rectangle m_Rec;
        
        public Block(float x, float y, int useTex)
        {
            m_Rec = new XNACS1Rectangle(new Vector2(x, y), BlockBreaker.BLOCK_WIDTH, BlockBreaker.BLOCK_HEIGHT);
            
            float rBase = 0.6f, cBase = 0.2f;
            float range = 0.8f;

            m_Rec.OutsideColor = new Color(new Vector3(rBase + range * XNACS1Base.RandomFloat(),
                                                    cBase + range * XNACS1Base.RandomFloat(),
                                                    cBase + range * XNACS1Base.RandomFloat()));
            m_Rec.CenterColor = new Color(new Vector3(rBase + range * XNACS1Base.RandomFloat(),
                                                   cBase + range * XNACS1Base.RandomFloat(),
                                                   cBase + range * XNACS1Base.RandomFloat()));

            m_Rec.Texture = useTex.ToString();
        }

        public Vector2 Position {
            set { m_Rec.Center = value; }
            get { return m_Rec.Center; } 
        }

        public bool CollideWithBall(Ball b) 
        {
            XNACS1Circle bc = b.GetCircle;

            if (!m_Rec.Collided(bc))
                return false;

            Vector2 v = b.Velocity;

            b.Bounce();
            if (bc.LeftOf(m_Rec) && (v.X > 0))
            {     // moving right-ward into the left side
                v.X = -v.X;
                bc.CenterX = m_Rec.MinBound.X - bc.Radius;
            }
            else if ((bc.RightOf(m_Rec)) && (v.X < 0))
            {   // moving left-ward into the right side
                v.X = -v.X;
                bc.CenterX = m_Rec.MaxBound.X + bc.Radius;
            }
            else
            {
                v.Y = -v.Y;
                if (bc.Above(m_Rec))
                    bc.CenterY = m_Rec.MaxBound.Y + bc.Radius;
                else
                    bc.CenterY = m_Rec.MinBound.Y - bc.Radius;
            }
            b.Velocity = v;            

            return true;
        }

        public virtual bool DeleteBlock(bool force)
        {
            m_Rec.RemoveFromAutoDrawSet();
            return true;
        }
    }
}