using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

// Our own library
using XNACS1Lib;


namespace BlockBreaker_Program
{

    class BlockSet
    {
        private const float BLOCK_PROBABLITY = 0.3f;

        private List<Block> m_TheSet = null;
        private int m_TotalBreakable = -1;

        private int m_TotalBreak = 0;

        public BlockSet()
        {
            InitializeBlocks();
        }

        public bool HasWon()
        {
            return (m_TotalBreak == m_TotalBreakable);
        }

        public void InitializeBlocks()
        {
            if (null != m_TheSet)
            {
                for (int c = 0; c < m_TheSet.Count; c++)
                    m_TheSet[c].DeleteBlock(true);
            }

            m_TotalBreak = 0;
            m_TotalBreakable = 0;
            m_TheSet = new List<Block>();
            int texNum = XNACS1Base.RandomInt(1, 8);
            for (int r = 0; r < BlockBreaker.NUM_BLOCK_ROWS; r++)
            {
                for (int c = 0; c < BlockBreaker.NUM_BLOCK_COL; c++)
                {
                    if (XNACS1Base.RandomFloat() > BLOCK_PROBABLITY)
                    {
                        float y = (r * BlockBreaker.BLOCK_HEIGHT) + BlockBreaker.LOWEST_BLOCK_Y;
                        float x = (c * BlockBreaker.BLOCK_WIDTH) + (BlockBreaker.BLOCK_WIDTH/2.0f);
                        Block b;
                        if (XNACS1Base.RandomFloat() > 0.25f)
                        {
                            b = new Block(x, y, texNum);
                            m_TotalBreakable++;
                        }
                        else
                            b = new SolidBlock(x, y, 0);
                        m_TheSet.Add(b);
                    }
                }
            }
        }

        public void BreakBlocks(Ball ball)
        {
            int c = m_TheSet.Count - 1;
            bool brokeOne = false;

            while ( (!brokeOne) && (c >= 0)) {
                if (m_TheSet[c].CollideWithBall(ball))
                {
                    if (m_TheSet[c].DeleteBlock(false))
                    {
                        m_TheSet.RemoveAt(c);
                        m_TotalBreak++;
                        brokeOne = true;
                        XNACS1Base.PlayACue("Break");
                    }
                    else
                    {
                        XNACS1Base.PlayACue("Bounce");
                    }
                }
                c--;
            }
            XNACS1Base.EchoToTopStatus("Total blocks left:" + m_TheSet.Count + " Total broke:" + m_TotalBreak);
        }
    }
}